import jsons
from askdata.smartquery import *

if __name__ == '__main__':
    # TEST SmartQuery Object
    field1 = Field(column='{{measure.A}}', aggregation="MAX", dataset='{{dataset.A}}', entityType="P_MEASURE", alias="max_measure")
    field2 = Field(column='{{dimension.A}}', dataset='{{dataset.B}}', entityType="P_DIMENSION")
    field3 = Field(column='{{timeDimension.A}}', dataset='{{dataset.C}}', entityType="P_TIMEDIM", aggregation="year")
    field4 = Field(column="{{unknownDateDimension.A}}")
    from1 = From('{{dataset.A}}')
    from2 = From('{{dataset.B}}')
    from3 = From('{{dataset.C}}')
    condition1 = Condition(field4, "FROM", direction="NEXT", steps="{{number.A}}", interval="{{timeDimension.B}}")
    condition2 = Condition(field1, "LOE", ["{{number.B}}"])
    condition3 = Condition(field2, "IN", ["{{entity.A}}"])
    condition4 = Condition(field4, "RANGE", value=["{{timePeriodStart.A}}", "{{timePeriodEnd.A}}"])
    sorting1 = Sorting("{{measure.A}}", SQLSorting.DESC)
    component = ChartComponent(type='chart', queryId="0", chartType='LINE')
    query1 = Query(fields=[field1, field2, field3], datasets=[from1, from2, from3], where=[condition1, condition2, condition3, condition4],
                   orderBy=[sorting1], limit=10)
    print(query1.to_sql())
    smartquery = SmartQuery(queries=[query1], components=[component])
    dump = jsons.dumps(smartquery)
    print(dump)
    smartquery = jsons.loads(dump, SmartQuery)
    print(jsons.dumps(smartquery, strip_nulls=True))
    print(smartquery)
    print(smartquery.queries[0].to_sql())
    # print("ORIGINAL JSON: ", dump)
    # compressed_json = SmartQuery.compress(dump)
    # print("COMPRESSED JSON: ", compressed_json)
    # decompressed_json = SmartQuery.decompress(compressed_json)
    # print("DECOMPRESSED JSON: ", decompressed_json)
    # print(str(dump) == decompressed_json)

    # TEST SPELL OUT
    # smartquery spell_out
    sq = SmartQuery(queries=[Query(
        fields=[Field(column="store_city", alias="Store City", internalDataType="STRING", sourceDataType="TEXT"),
                Field(column="time_the_date", alias="Time", internalDataType="DATE", sourceDataType="TEXT")],
        where=[Condition(Field(column="time_the_date", aggregation="MONTH", alias="Time", internalDataType="DATE", sourceDataType="TEXT"),
                         operator="FROM", direction="NEXT", steps="2"),
               Condition(Field(column="time_the_date", aggregation="MONTH", alias="Time", internalDataType="DATE",
                               sourceDataType="TEXT"),
                         operator="FROM", value=["2022"]),
               Condition(Field(column="units shipped", aggregation="MAX", alias="max_units_shipped", internalDataType="NUMERIC",
                               sourceDataType="REAL"),
                         operator="GOE", value=["100"])
               ],
        orderBy=[Sorting(field="Store City", order=SQLSorting.DESC),
                 Sorting(field="Time", order=SQLSorting.DESC)],
        limit=8,
        offset=0
    )])
    print(sq.spell_out())



from askdata.smartgraph import smart_title, smart_substitution
import threading
import time

if __name__ == "__main__":
    smartquery = {"components": [{"type": "chart", "queryId": "q0", "chartType": "LINE"}],
                  "queries": [{"datasets": [], "fields": [
                      {"aggregation": "MAX", "column": "totale_casi", "internalDataType": "NUMERIC"},
                      {"aggregation": "AVG", "column": "deceduti", "internalDataType": "NUMERIC"},
                      {"column": "denominazione_regione", "internalDataType": "STRING"},
                      {"column": "monthly", "internalDataType": "DATE"}], "id": "q0", "limit": "3",
                               "where": [{"field": {"column": "deceduti"}, "operator": "GOE", "value": ["30"]},
                                         {"field": {"column": "denominazione_regione"}, "operator": "IN",
                                          "negate": True, "value": ["Lazio"]},
                                         {"field": {"column": "monthly"}, "operator": "RANGE",
                                          "value": ["2020-07-13", "2020-10-16"]},
                                         {"field": {"column": "monthly"}, "operator": "RANGE", "direction": "CURR",
                                          "steps": "2"}]}]}
    metadata = {"totale_casi": "Totale casi", "deceduti": "Deceduti", "denominazione_regione": "Regione",
                "monthly": "Monthly"}

    sentence = "monuments <mask> Italy"

    thread_list = []
    t = threading.Thread(target=smart_title, args=[smartquery, metadata, "en-US"])
    t.start()
    thread_list.append(t)
    t = threading.Thread(target=smart_substitution, args=[sentence])
    t.start()
    thread_list.append(t)
    print(f'Active Threads: {threading.active_count()}')
    for t in thread_list:
        t.join()
    print(f'Active Threads: {threading.active_count()}')



from askdata.smartgraph import smart_substitution

if __name__ == "__main__":

    sentence = "monuments <mask> Italy"
    new_sentence = smart_substitution(sentence=sentence, language="en")
    print(new_sentence)



from askdata.human2query import nl2query

if __name__ == "__main__":
    nl = 'frotok {{timePeriods.timePeriodA.value}} {{measures.measureA.name}} {{measures.measureB.name}} {{timeDimensions.timeDimensionA.value}} limit by {{numbers.numberA.value}} lines associated with {{measures.measureA.name}} in from {{numbers.numberB.value}} to as {{numbers.numberC.value}} when {{entityTypes.entityTypeA.name}} by {{timeDimensions.timeDimensionB.value}} limited {{numbers.numberD.value}}'
    smartquery, version = nl2query(nl, language="en-US", model_version="2022.06.08")
    print(smartquery)
    print(version)



import requests
import json
import pprint
from askdata.askdata_client import *

### function for testing login, this function is used inside run_authentication_test and return the response
def test_login(username, password, domain):
    url = "https://api.askdata.com/security/domain/" + domain + "/oauth/token"

    payload = "grant_type=password&username=" + username + "&password=" + password

    headers = {'authority': "api.askdata.com",
               'accept': "application/json, text/plain, */*",
               'accept-language': "it-IT,it;q=0.9,en-US;q=0.8,en;q=0.7,ar;q=0.6",
               'authorization': "Basic YXNrZGF0YS1zZGs6YXNrZGF0YS1zZGs=", 'content-type': "application/x-www-form-urlencoded", 
               'origin': "https://app.askdata.com",
               'referer': "https://app.askdata.com/",
               'sec-ch-ua-mobile': "?1",
               'sec-ch-ua-platform': "Android", 
               'sec-fetch-dest': "empty", 
               'sec-fetch-mode': "cors", 
               'sec-fetch-site': "same-site", 
               'user-agent': "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Mobile Safari/537.36"}

    response = requests.request("POST", url, data=payload, headers=headers)

    return response
    
    
### function for testing login saving an html message    
def run_authentication_test(credentials_to_test):
    
    #the use of global variables allows to save them outside the function without using it again as input/output
    global message_body, total_passed, total_failed
    
    # initialize the html message - the xxxx will be replace at the end of this function with the number of tests passed
    
    message_body = ""

    message_body += "<h1>Authentication Testing</h1> Tests passed: xxxxx/" + str(len(credentials_to_test.keys())) + " ------- Fails: yyyy %" + "\n \n "

    # these counters are used for calculate the % of passed/failed test
    
    # total_passed, total_failed are used for the OVERALL notebook, i.e. it counts all tests and it's needed for make the % of all tests passed to use in the mail object
    
    # auth_test_passed, auth_test_failed are used only for the authentication test
    
    total_passed, total_failed, auth_test_passed, auth_test_failed = 0, 0, 0, 0

    
    #test the authentication for each user
    
    for user in credentials_to_test:

        psw = credentials_to_test[user][0]

        domain = credentials_to_test[user][1]


        ## if the authentication for that user passed (i.e. the response status of the test_login function is 200), then update the html message with passed user and increase the passed counters
        try:
            assert test_login(user, psw, domain).status_code == 200

            message_body += "<br><b style='color:green;'>Login Successful</b> for " + user + " in domain " + credentials_to_test[user][1] + " \n "

            auth_test_passed += 1
            
            total_passed += 1

        ## if the authentication does not passed, update the html message with the failed user and increase the failed counters
        except AssertionError:

            message_body += "<br><b style='color:red;'>Login Failed</b> for " + user + " in domain " + credentials_to_test[user][1]

            auth_test_failed += 1
            
            total_failed += 1
            
    ## replace the xxxx and yyyy in the message body with the auth_test_passed and the % of test failed
            
    message_body = message_body.replace("xxxxx", str(auth_test_passed)).replace("yyyy", str(round(auth_test_failed*100 / (auth_test_failed + auth_test_passed), 2)))
    

    return message_body


def get_message_body():
    
    return message_body



## initialize testing for specific Agent given user and psw or using the token, needed to pass the agent_slug too
## this function inizialite the counters and the html message for a specific agent. It also login on that  agent
def initialize_agent_test(username = '', password = '', domain = 'askdata', token = None, agent_slug = ''):
    
    global message_body, agent_id, test_passed, test_failed, glob_token
    
    ## set counters for tests passed and failed in specific agent
    test_passed, test_failed = 0, 0
    
    
    #get the token as global variable so no needed to authenticate again in the function run_query
    if token:
        askdata = Askdata(token = token, env = 'prod')
        
        glob_token = token
        
    else:
        askdata = Askdata(username = username, password = password, domainlogin = domain, env = 'prod')
            
        glob_token = test_login(username = username, password = password,  domain = domain).json()["access_token"]

    agent = askdata.agent(agent_slug)

    agent_name = agent._agent_name

    agent_id = agent._agentId

    message_body = get_message_body()

    ## update the previous html message of the authentication with the info of the agent
    message_body += "<h2>AGENT {}</h2> \n <i>https://app.askdata.com/{}</i> <b>id</b>: {} <br>Tests passed: xxxxx ------- Fails: yyyy % \n \n ".format(agent_name, agent_slug, agent_id)
    
    return message_body
    
    
def get_agent_id():
    
    return agent_id

def get_test_passed():
    
    return test_passed

def get_test_failed():
    
    return test_failed

def get_total_passed():
    
    return total_passed

def get_total_failed():
    
    return total_failed

def get_glob_token():
    
    return glob_token
    
    
    
### function for running queries used inside the function run_test. It results in the response of the query
def run_query(token, agent_id, query):
    
    url = "https://api.askdata.com/smartfeed/askdata/preflight"

    querystring = {"agentId": agent_id, "lang":"en"}

    payload = "{\"text\":\"" + query + " \"}"

    headers = {'authority': "api.askdata.com", 'accept': "application/json, text/plain, */*", 
               'accept-language': "it-IT,it;q=0.9,en-US;q=0.8,en;q=0.7,ar;q=0.6", 'authorization': "Bearer " + token,      
               'content-type': "application/json",      
               'origin': "https://app.askdata.com",      
               'referer': "https://app.askdata.com/",      
               'sec-ch-ua-mobile': "?1",      
               'sec-ch-ua-platform': "Android",      
               'sec-fetch-dest': "empty",      
               'sec-fetch-mode': "cors",      
               'sec-fetch-site': "same-site",      
               'user-agent': "Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Mobile Safari/537.36"}


    response = requests.request("POST", url, data=payload, headers=headers, params=querystring)

    return response
  
    
    
#check the number of cards
def num_of_cards(response): 
    
    return (len(response.json()))


#check the component type in a specific position
def check_component(response, comp = 0):
    
    return response.json()[0]['attachment']['body'][comp]['component_type']


#check the name of the column in a table component in a specific position
def check_col_names(response, comp = 0):
    
    return set(response.json()[0]['attachment']['body'][comp]['details']['columns'])


#check column filtered, the operator and its value in a specific position (i.e. country in Italy)
def check_filter(response, comp = 0, filter_number = 0):
    
    return response.json()[0]['attachment']['body'][comp]['details']['filters'][filter_number]['field'], response.json()[0]['attachment']['body'][comp]['details']['filters'][filter_number]['operator'], response.json()[0]['attachment']['body'][comp]['details']['filters'][filter_number]['values']


#check if a button component has a specific label and link
def check_button(response, comp = 0):
    
    return response.json()[0]['attachment']['body'][comp]['details']['label'], response.json()[0]['attachment']['body'][comp]['details']['link']


## run the query test
## run the query test
def run_test(query = '',
             n_cards = None, check_component_types = None, 
             check_table_columns = None, check_filters_existance = None,
             check_button_existace = None,
           
            print_response = False):
    
    global message_body, test_passed, test_failed, total_passed, total_failed, glob_token
    
    agent_id = get_agent_id()
    
    message_body = get_message_body()
    
    total_passed = get_total_passed()
    
    total_failed = get_total_failed()
    
    test_passed = get_test_passed()
    
    test_failed = get_test_failed()
    
    glob_token = get_glob_token()
    
    response = run_query(glob_token, agent_id, query)
    
    
    failed_message = ""
    
    failed = False
    
    
    #failed = dict()
    
    #check the status response
    if response.status_code == 200:
        
        print('Response 200')
        
        if 'text' in response.json()[0]['attachment']['body'][0]['details'] and response.json()[0]['attachment']['body'][0]['details']['text'] == 'Could you be more specific?':
            
            print('Card <Could you be more specific?>')
            
            failed = True
            
            failed_message += "<br><b>Card returned:</b> Could you be more specific?\n"
            
        elif 'text' in response.json()[0]['attachment']['body'][0]['details'] and response.json()[0]['attachment']['body'][0]['details']['text'] == 'There are no cards in this channel. You can share existing cards on this channel, or create new ones using Feed Rules!':
            
            print('Card <There are no cards in this channel. You can share existing cards on this channel, or create new ones using Feed Rules!>')
            
            failed = True
            
            failed_message += "<br><b>Card returned:</b> There are no cards in this channel. You can share existing cards on this channel, or create new ones using Feed Rules!\n"
            
        else:
            #check the number of cards
            if n_cards:

                if num_of_cards(response) == n_cards:
                    print('\n')
                    print('Number of cards expected passed')


                else:

                    failed = True

                    #failed['Number or cards'] = num_of_cards(response)

                    failed_message += "<br><b>Expected number of cards</b>: {}, but given {}".format(n_cards, num_of_cards(response))
                        
                
                
            #check comp types
    
            if check_component_types:

                print('\n')

                for position, component in enumerate(check_component_types):

                    if check_component(response, position) == component:

                        print('Check for component {} passed'.format(component))

                    else:

                        failed = True

                        print('Check for Components failed. Expected {}, but given {}'.format(component, check_component(response, position)))


                        failed_message += "<br><b>Expected component in position {}</b>: \"{}\", but given \"{}\".".format(position, component, check_component(response, position))
            
    
            #check tab cols
    
            if check_table_columns:

                print('\n')

                for position, columns in enumerate(check_table_columns):

                    if columns != []:  
                        
                  

                        if response.json()[0]['attachment']['body'][position]['component_type'] == 'table' and check_col_names(response, position) == set(columns):

                            print('Check for column names in table passed')


                        else:

                            failed = True

                            print('Check for column names in table failed. Expected {} but given {}'.format(columns, list(check_col_names(response, position))))


                            failed_message += "<br><b>Expected columns</b>: <pre>{}</pre>".format(pprint.pformat(columns))
                                
                    
        
            if check_filters_existance:
            
                for position, filters in enumerate(check_filters_existance):

                    if filters != []:

                        for filter_number, f in enumerate(filters):

                            if check_filter(response, position, filter_number) == f:

                                print('Check for filters {} passed'.format(f))

                            else:

                                failed = True

                                print('Check for filters {} failed, given {}'.format(f, check_filter(response, position, filter_number)))

                                failed_message += "<br><b>Expected filters</b>: <pre>{}</pre> but given <pre>{}</pre>".format(pprint.pformat(f), pprint.pformat(check_filter(response, position, filter_number)))
        
        
            if check_button_existace:
            
                for position, button in enumerate(check_button_existace):

                    if button != []:

                        if list(check_button(response, position)) == button:

                            print('Check for button passed')

                        else:

                            failed = True

                            print('Check for button failed. Expected {}, but given {}'.format(button, list(check_button(response, position))))

                            failed_message += "<br><b>Expected button</b>: <pre>{}</pre> but given <pre>{}</pre>".format(pprint.pformat(button), pprint.pformat(list(check_button(response, position))))
    
    
    else:
    
    
        failed = True
        
        status = response.status_code
        
        print('Response', status)
        
        #failed['Status Response'] = status
        
        failed_message += "<br><b>Response status</b> {}".format(status)
    
    
    
    if not failed:
    
        test_passed += 1
        
        total_passed += 1
        
        print('All Check Passed\n')
        
        message_body += "<br><b style='color:green;'>Test Successful</b> for query: <i>{}</i>".format(query)
        
    else:
    
        test_failed += 1
        
        total_failed += 1
        
        print('Test Failed\n')
        
        message_body += "<br><b style='color:red;'>Test Failed</b> for query: <i>{}</i>".format(query)
        
        message_body += failed_message
        
    
    try:
        card_0 = response.json()[0]['attachment']['body']

        for i in range(len(card_0)):

            if card_0[i]['component_type'] == 'table':

                message_body += "<br><b>Data Response (Row 1):\n</b><pre>" + pprint.pformat(card_0[i]['details']['columns']) + '\n</pre>'
                message_body += "<pre>" + pprint.pformat(card_0[i]['details']['rows'][0]) + '\n</pre>'

    except:
        
        pass

    
    if print_response:
        
    #this print is helpful for debugging code
    
        print("Response:")
        
        try:
            print(json.dumps(response.json()[0]['attachment']['body'], 
          
          sort_keys = False, indent = 4))
            
        except:
            
            pass
    
    return message_body

## finalize agent test message body after running one o more run_test

def finalize_agent_test():
    
    global test_passed, test_failed, message_body
    
    test_passed = get_test_passed()

    test_failed = get_test_failed()
    
    message_body = get_message_body()
    
    message_body = message_body.replace("xxxxx", str(test_passed) + "/" + str((test_failed + test_passed)) ).replace("yyyy", str(round(test_failed*100, 2) / (test_failed + test_passed)))
    
    return message_body



#required !pip install googleads -q


import pandas as pd
import numpy as np

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.protobuf.json_format import MessageToDict



import _locale


"""
MANDATORY INPUT:

start_date, end_date as string "yyyy-mm-dd"

OPTIONAL INPUTS: 



- yaml_file_path: the path of 'googleads.yaml'. 
                Optional, but mandatory to have it in same folder script if used the default parameter.

"""

def get_google_ads(start_date, end_date, admin_account,
                   
                   
                   ## optional
                   
                   yaml_file_path = 'googleads.yaml'
                  ):

    _locale._getdefaultlocale = (lambda *args: ['en_US', 'UTF-8'])
    
    #googleads_client = GoogleAdsClient.load_from_storage('googleads.yaml')
    googleads_client = GoogleAdsClient.load_from_storage(yaml_file_path)
    
    ga_service = googleads_client.get_service("GoogleAdsService")
    
    query = f"""
            SELECT 
            campaign_budget.amount_micros,
            campaign.id, 
            customer.id,
            customer.descriptive_name,
            customer.currency_code,
            metrics.cost_micros, 
            segments.month, 
            segments.date, 
            campaign.name, 
            metrics.impressions, 
            metrics.clicks, 
            metrics.ctr, 
            metrics.average_cpc, 
            metrics.conversions, 
            metrics.view_through_conversions, 
            metrics.cost_per_conversion, 
            campaign.status, 
            metrics.all_conversions_from_interactions_rate, 
            metrics.average_cpm 
            FROM campaign 
            WHERE 
            segments.date BETWEEN '{start_date}' AND '{end_date}' """
    
    search_request = googleads_client.get_type("SearchGoogleAdsStreamRequest")
        
    
    google_ads = pd.DataFrame(columns = ['Customer ID','Brand','Currency','Campaign state','Campaign','Campaign ID','Clicks','View-through conv.','Conversions','Cost','Cost / conv.','CTR','Conv. rate','Avg. CPC','Avg. CPM','Impressions','Budget','Day','Month'])
        
    df = pd.DataFrame(columns = ['Customer ID','Brand','Currency','Campaign state','Campaign','Campaign ID','Clicks','View-through conv.','Conversions','Cost','Cost / conv.','CTR','Conv. rate','Avg. CPC','Avg. CPM','Impressions','Budget','Day','Month'])
        
    for key, acc_id in admin_account.items():

        
        
        stream = ga_service.search(customer_id = acc_id, query=query)
        
        
        
        st = next(stream.pages)
        dictobj = MessageToDict(st)
        
        #print(dictobj)
        
        if 'results' in dictobj:
            
            df = pd.json_normalize(dictobj,record_path=['results'])


            df = df.drop(columns = ['customer.resourceName', 'campaign.resourceName', 'campaignBudget.resourceName'])


            df = df.rename(columns =  {'customer.id':'Customer ID',
                                     'customer.descriptiveName':'Brand',
                                     'customer.currencyCode':'Currency',
                                     'campaign.status':'Campaign state',
                                     'campaign.name':'Campaign',
                                     'campaign.id':'Campaign ID',
                                     'metrics.clicks':'Clicks',
                                     'metrics.viewThroughConversions':'View-through conv.',
                                     'metrics.conversions':'Conversions',
                                     'metrics.costMicros':'Cost',
                                     'metrics.costPerConversion':'Cost / conv.',
                                     'metrics.ctr':'CTR',
                                     'metrics.allConversionsFromInteractionsRate':'Conv. rate',
                                     'metrics.averageCpc':'Avg. CPC',
                                     'metrics.averageCpm':'Avg. CPM',
                                     'metrics.impressions':'Impressions',
                                     'campaignBudget.amountMicros':'Budget',
                                     'segments.date':'Day',
                                     'segments.month':'Month'} )

            

            #print(df.head())
            print(f'Extracted data for {key}')

        else:
            print(f'No data for {key}')

        google_ads = pd.concat([google_ads, df])
    
        
    google_ads = google_ads[['Month', 'Day', 'Campaign ID', 'Customer ID', 'Campaign',
           'Campaign state', 'Budget', 'Currency', 'Clicks', 'Impressions', 'CTR',
           'Avg. CPC', 'Cost', 'Conversions', 'View-through conv.', 'Cost / conv.',
           'Conv. rate', 'Avg. CPM', 'Brand']].reset_index(drop = True)
    
    to_float = ['Clicks', 'Impressions', 'Cost', 'CTR', 'View-through conv.', 'Conversions', 'Cost / conv.', 'Conv. rate', 'Avg. CPC', 'Avg. CPM']

    google_ads['Campaign ID'] = google_ads['Campaign ID'].astype(int)

    for c in to_float:

        google_ads[c] = google_ads[c].astype(float)

    # micro amount 1000000
    google_ads['Cost'] = google_ads['Cost']/1000000
    
    google_ads['CTR'] = round(google_ads['CTR']*100, 2).astype(str) + '%'
    
    google_ads['Conv. rate'] = round(google_ads['Conv. rate']*100, 2).astype(str) + '%'
    
    google_ads['Campaign state'] = google_ads['Campaign state'].str.lower()
    
    
    google_ads['Avg. CPC'] = round(google_ads['Avg. CPC'], 1)
    google_ads['Avg. CPM'] = round(google_ads['Avg. CPM'], 1)
    google_ads['Cost / conv.'] = round(google_ads['Cost / conv.'], 1)
        
    return google_ads


import pandas as pd
from askdata.human2query import nlp, smartner_automated_analysis
import time

if __name__ == "__main__":

    # ---------------------------------------------------------------------------------------------------------------- #
    print("#--------------------------SmartNER Online Test--------------------------#")

    # Prod
    # token = ""
    # datasets = [""]
    # language = "en"
    # env = "prod"

    # Dev
    token = ""
    datasets = ["7598d843-b208-47c3-b284-b203d0f8b2bc-CSV-71c3bba3-35e6-45d0-be14-bb39f2a20e4c"]
    language = "en"
    env = "dev"

    # Usage
    nl = "units"
    df = pd.read_excel("./dataset.xlsx")

    start = time.time()
    smartquery_list = nlp(nl=nl, token=token, datasets=datasets, response_type="query2sql")
    end = time.time()

    print("SmartNER  Online")
    for sq in smartquery_list:
        print(sq)
        print()

    print("Time: ", end - start, "s")
    print()

    # ---------------------------------------------------------------------------------------------------------------- #
    print("#--------------------------SmartNER Offline Test--------------------------#")

    # Usage
    nl = "units ordered"
    df = pd.read_excel("./inventory.xlsx")

    start = time.time()
    smartquery_list = nlp(nl=nl, dataframe=df)
    end = time.time()

    print("SmartNER Offline")
    for sq in smartquery_list:
        print(sq)
        print()

    print("Time: ", end - start, "s")
    print()

    # ---------------------------------------------------------------------------------------------------------------- #
    # print("#--------------------------SmartNER Automated Test--------------------------#")

    # suggestions = [["{{dimension.A}} and {{measure.A}}", {"{{measure.A}}": {"code": "DOWNLOAD", "value": "Download", "dataset": "f2c705dd-f63f-475b-95a9-c1ad20f33716-MYSQL-70c82ef8-2cb2-40dd-a92f-b49a80d0a305"}, "{{dimension.A}}": {"code": "WEB SITE", "value": "Web site", "dataset": "f2c705dd-f63f-475b-95a9-c1ad20f33716-MYSQL-70c82ef8-2cb2-40dd-a92f-b49a80d0a305"}}, ["f2c705dd-f63f-475b-95a9-c1ad20f33716-MYSQL-70c82ef8-2cb2-40dd-a92f-b49a80d0a305"]]]
    #
    # start = time.time()
    # smartquery_list = smartner_automated_analysis(suggestions, token, env="dev", response_type="deanonymize")
    # end = time.time()
    #
    # print("Automated analysis")
    # for sq in smartquery_list:
    #     print(sq)
    #     print()
    #
    # print("Time: ", end - start, "s")



import json
import jsons
import operator
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import random
import logging
import requests
import pandas as pd
import os
import yaml
import warnings

warnings.simplefilter(action='ignore', category=FutureWarning)

ops = {
    '<': operator.lt,
    '<=': operator.le,
    '==': operator.eq,
    '!=': operator.ne,
    '>=': operator.ge,
    '>': operator.gt
}

root_dir = os.path.abspath(os.path.dirname(__file__))
# retrieving base url
yaml_path = os.path.join(root_dir, '../askdata/askdata_config/base_url.yaml')
with open(yaml_path, 'r') as file:
    # The FullLoader parameter handles the conversion from YAML
    # scalar values to Python the dictionary format
    url_list = yaml.load(file, Loader=yaml.FullLoader)


def ask_dataframe(dataframe: pd.DataFrame, query: str):
    human2sql_request = {
        "dataframe": dataframe.to_dict(),
        "query": query
    }

    human2sql_url = url_list['BASE_URL_HUMAN2SQL_DEV']

    human2sql_response = requests.post(human2sql_url, json=human2sql_request)
    response_df = []
    if human2sql_response.ok:
        res = human2sql_response.json()
        if 'result' in res:
            all_dfs = res['result']
            for df in all_dfs:
                response_df.append(pd.DataFrame(df))
        if 'messages' in res and res['messages']:
            for mex in res['messages']:
                print(mex)
    else:
        print("Error: " + str(human2sql_response))

    return response_df


def get_conditional_phrases(conditions, phrase1, phrase2):
    bool_array = []
    for condition in conditions:
        for op in ops:
            if op in condition:
                splitted = condition.split(' ' + op + ' ')
                operator = ops[op]
                bool_array.append(operator(splitted[0], splitted[1]))
        # bool_array.append(eval(condition))
    if False in bool_array:
        return phrase2
    else:
        return phrase1


def get_random_synonymous(synonyms):
    random.seed()
    nKeys = len(synonyms.keys())
    pickedItem = random.randint(0, nKeys - 1)
    synPicked = synonyms[pickedItem]
    return synPicked


def words_to_digits(phrase):
    words = phrase.split(" ")
    num = []
    for word in words:
        if "." in word:
            word = word.replace(".", "")
        if "," in word:
            word = word.replace(",", "")
        if word.isdigit():
            num.append(int(word))
    return num


def add_random_synonymous_to_sentence(phrase, placeholder, synonyms):
    random.seed()
    nKeys = len(synonyms.keys())
    pickedItem = random.randint(0, nKeys - 1)
    synPicked = synonyms[pickedItem]
    phrase = phrase.replace(placeholder, synPicked)
    return phrase


def data2nl(df, base_sentence=None, request=None, table_description=False):
    # The parameter "df" is a dataframe, but it will be converted into a dict, and called df in each case

    if request is not None:
        table_description = True
    if base_sentence is not None:
        table_description = False

    if not df.empty:
        headers = {
            "Content-Type": "application/json"
        }
        s = requests.Session()
        s.keep_alive = False
        retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
        s.mount('https://', HTTPAdapter(max_retries=retries))

        url = url_list['BASE_URL_DATA2NL_DEV'] + "/data2nl"

        dict_df = df.to_dict(orient='records')

        if table_description is False:
            response_df = pd.DataFrame()

            for el in dict_df:

                row = "Data: " + str(el)

                if base_sentence is not None:
                    row = "BaseString: " + base_sentence + "\n" + row

                data = {"data": row}

                r = s.post(url=url, headers=headers, json=data)
                r.raise_for_status()

                try:
                    nl = r.json()['nl']
                    el['nl'] = nl
                except Exception as e:
                    logging.error(str(e))
                    el['generated_nl'] = ""
                tmp_df = pd.DataFrame()
                tmp_df = tmp_df.append(el, ignore_index=True)
                response_df = pd.concat([response_df, tmp_df], ignore_index=True, axis=0)
            return response_df
        else:
            row = "Table: " + str(dict_df)

            if request is not None:
                row = row + "\n" + "Request:\'" + request + "\'"
            data = {"data": row}

            r = s.post(url=url, headers=headers, json=data)
            r.raise_for_status()

            try:
                nl = r.json()['nl']
            except Exception as e:
                logging.error(str(e))
                nl = ""
            return nl

    else:
        print("Input DataFrame is empty!")
        df_response = pd.DataFrame()
        return df_response


def query2sql(smartquery, driver, model_version=None):
    # Google Pod
    headers = {
        "Content-Type": "application/json"
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/query_to_sql"

    stringed_smartquery = jsons.dumps(smartquery, strip_nulls=True)
    smartquery = json.loads(stringed_smartquery)

    json_data = {
        "smartquery": smartquery,
        "driver": driver
    }

    # Params check
    params = {}

    if model_version is not None:
        params["model_version"] = model_version

    r = s.post(url=url, headers=headers, json=json_data, params=params)
    r.raise_for_status()

    try:
        response = r.json()
        return response["sql"]
    except Exception as e:
        logging.error(str(e))
        return []


def nl2query(nl, language="en-US", model_version=None):
    # Google Pod
    headers = {
        "Content-Type": "application/json"
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    url = url_list['BASE_URL_NL2QUERY_DEV'] + "/query"

    if "it" in language:
        language = "it-IT"
    else:
        language = "en-US"

    json_data = {
        "nl_ner": nl,
        "lang": language
    }

    # Params check
    params = {}

    if model_version is not None:
        params["model_version"] = model_version

    r = s.post(url=url, headers=headers, json=json_data, params=params)
    r.raise_for_status()

    try:
        response = r.json()
        smartquery, version = response['smartquery'], response['model_version']
        return smartquery, version
    except Exception as e:
        logging.error(str(e))
        print("!-!-! Failed SmartQuery object creation. !-!-!")
        return None, None


def complex_field_calculator(smartquery, driver):
    # Google Pod
    headers = {
        "Content-Type": "application/json"
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/query_to_sql"

    stringed_smartquery = jsons.dumps(smartquery, strip_nulls=True)
    smartquery = json.loads(stringed_smartquery)

    data = {
        "smartquery": smartquery,
        "driver": driver
    }

    r = s.post(url=url, headers=headers, json=data)
    r.raise_for_status()

    try:
        dict_response = r.json()
        sql = dict_response['sql']
        return sql
    except Exception as e:
        logging.error(str(e))


def complex_filter_calculator(smartquery, driver):
    # Google Pod
    headers = {
        "Content-Type": "application/json"
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    url = url_list['BASE_URL_QUERY2SQL_DEV'] + "/query_to_sql"

    stringed_smartquery = jsons.dumps(smartquery, strip_nulls=True)
    smartquery = json.loads(stringed_smartquery)

    data = {
        "smartquery": smartquery,
        "driver": driver
    }

    r = s.post(url=url, headers=headers, json=data)
    r.raise_for_status()

    try:
        dict_response = r.json()
        sql = dict_response['sql']
        return sql
    except Exception as e:
        logging.error(str(e))


def nlp(nl, language="en", response_type="deanonymize", token="", env="dev", online=True, datasets=None, dataframe=None,
        use_ner=None, use_cache=None, cache_mode=None, use_redis=None, use_specific_ner=None,
        smartner_model_version=None, nl2query_model_version=None, query2sql_model_version=None,
        similarity_threshold=None):

    # Check mode online/offline
    if datasets is not None and dataframe is not None:
        online = True
    elif dataframe is not None:
        online = False
        response_type = "query2sql"
    elif datasets is not None:
        online = True

    if online is True and not (nl != "" and datasets is not None and token != ""):
        error = "!-!-! Online mode requires these inputs: natural language query, datasets IDs and authorization token."
        print(error)
        return []
    elif online is False and not (nl != "" and dataframe is not None):
        error = "!-!-! Offline mode requires these inputs: natural language query and dataframe."
        print(error)
        return []

    headers = {
        "Content-Type": "application/json",
        "authorization": "Bearer " + token
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    if env == "dev":
        url = url_list['BASE_URL_SMARTNER_DEV'] + "/query"
    else:
        url = url_list['BASE_URL_SMARTNER_PROD'] + "/query"

    if dataframe is not None:
        dataframe = dataframe.fillna('')
        date_format = '%Y-%m-%d %H:%M:%S'
        for c in dataframe.columns:
            if "datetime" in str(dataframe[c].dtype):
                dataframe[c] = dataframe[c].dt.strftime(date_format)
        dataframe = dataframe.to_dict()

    json_data = {
        "nl": nl,
        "lang": language,
        "datasets": datasets,
        "dataframe": dataframe
    }

    # Params check
    params = {}

    if use_cache is not None:
        if use_cache not in [True, False]:
            use_cache = False
            print("!-!-! Value of use_cache parameter not allowed. Setting to False.")
        params["use_cache"] = use_cache

    if cache_mode is not None:
        if cache_mode not in ["standard", "similarity", "only_cache"]:
            cache_mode = "standard"
            print("!-!-! Value of similarity_threshold parameter not allowed. Setting to 85%.")
        params["cache_mode"] = cache_mode

    if use_specific_ner is not None:
        if use_specific_ner not in [True, False]:
            use_specific_ner = False
            print("!-!-! Value of use_specific_ner parameter not allowed. Setting to False.")
        params["use_specific_ner"] = use_specific_ner

    if similarity_threshold is not None:
        if not (type(similarity_threshold) == int or type(similarity_threshold) == float):
            similarity_threshold = 0.85
            print("!-!-! Value of similarity_threshold parameter not allowed. Setting to 85%.")
        params["similarity_threshold"] = similarity_threshold

    if response_type is not None:
        if response_type not in ["anonymize", "nl2query", "deanonymize", "query2sql"]:
            response_type = "deanonymize"
            print("!-!-! Value of response_type parameter not allowed. Setting to deanonymize.")
        params["response_type"] = response_type

    if use_ner is not None:
        if use_ner not in [True, False]:
            use_ner = True
            print("!-!-! Value of use_ner parameter not allowed. Setting to True.")
        params["use_ner"] = use_ner

    # For now
    if use_redis is not None:
        if use_redis not in [True, False]:
            use_redis = False
            print("!-!-! Value of use_redis parameter not allowed. Setting to False.")
        elif use_redis is True:
            use_redis = False
            print("!-!-! Warning: use_redis parameter not allowed for now. Setting to False.")
        params["use_redis"] = use_redis

    if online not in [True, False]:
        online = True
        print("!-!-! Value of online parameter not allowed. Setting to True.")
    params["online"] = online

    if smartner_model_version is not None:
        params["smartner_model_version"] = smartner_model_version
    if nl2query_model_version is not None:
        params["nl2query_model_version"] = nl2query_model_version
    if query2sql_model_version is not None:
        params["query2sql_model_version"] = query2sql_model_version

    r = s.post(url=url, headers=headers, json=json_data, params=params)
    r.raise_for_status()

    try:
        response = r.json()

        if not online:
            df_response = []
            for dt in response["results"]:
                df_response.append(pd.DataFrame.from_dict(dt))
            response["results"] = df_response
            del df_response

        return response["results"]
    except Exception as e:
        print(e)
        return []


def smartner_automated_analysis(suggestions, token, env="dev", response_type="deanonymize"):
    headers = {
        "Content-Type": "application/json",
        "authorization": "Bearer " + token
    }

    s = requests.Session()
    s.keep_alive = False
    retries = Retry(total=5, backoff_factor=1, status_forcelist=[502, 503, 504])
    s.mount('https://', HTTPAdapter(max_retries=retries))

    if env == "dev":
        url = url_list['BASE_URL_SMARTNER_DEV'] + "/automated_analysis"
    else:
        url = url_list['BASE_URL_SMARTNER_PROD'] + "/automated_analysis"

    json_data = {
        "suggestions": suggestions
    }

    # Params check
    params = {}

    if response_type in ["anonymize", "nl2query", "deanonymize", "query2sql"]:
        params["response_type"] = response_type

    r = s.post(url=url, headers=headers, json=json_data, params=params)
    r.raise_for_status()

    try:
        response = r.json()
        return response["results"]
    except Exception as e:
        logging.error(str(e))
        return []
